@@ -1,11 +1,78 @@
+import time
 from functools import partial
+from typing import Any, Iterable, Tuple, Union
+
 import numpy as np
 
 from qcodes import VisaInstrument
-from qcodes.instrument.parameter import ArrayParameter
-from qcodes.utils.validators import Numbers, Ints, Enum, Strings
+from qcodes.instrument.parameter import (
+    ArrayParameter,
+    Parameter,
+    ParameterWithSetpoints,
+    ParamRawDataType,
+)
+from qcodes.utils.validators import Arrays, ComplexNumbers, Enum, Ints, Numbers, Strings
 
-from typing import Tuple
+
+class ChannelTrace(ParameterWithSetpoints):
+    """
+    Parameter class for the two channel buffers
+    """
+
+    def __init__(self, name: str, channel: int, **kwargs: Any) -> None:
+        """
+        Args:
+            name: The name of the parameter
+            channel: The relevant channel (1 or 2). The name should
+                should match this.
+        """
+        super().__init__(name, **kwargs)
+
+        self._valid_channels = (1, 2)
+
+        if channel not in self._valid_channels:
+            raise ValueError('Invalid channel specifier. SR830 only has '
+                             'channels 1 and 2.')
+
+        if not isinstance(self.root_instrument, SR830):
+            raise ValueError('Invalid parent instrument. ChannelBuffer '
+                             'can only live on an SR830.')
+
+        self.channel = channel
+        self.update_unit()
+
+    def update_unit(self) -> None:
+        assert isinstance(self.root_instrument, SR830)
+        params = self.root_instrument.parameters
+        if params[f'ch{self.channel}_ratio'].get() != 'none':
+            self.unit = '%'
+        else:
+            disp = params[f'ch{self.channel}_display'].get()
+            if disp == 'Phase':
+                self.unit = 'deg'
+            else:
+                self.unit = 'V'
+            self.label = disp
+
+    def get_raw(self) -> ParamRawDataType:
+        """
+        Get command. Returns numpy array
+        """
+        assert isinstance(self.root_instrument, SR830)
+        N = self.root_instrument.buffer_npts()
+        if N == 0:
+            raise ValueError('No points stored in SR830 data buffer.'
+                             ' Can not poll anything.')
+
+        # poll raw binary data
+        self.root_instrument.write(f'TRCL ? {self.channel}, 0, {N}')
+        rawdata = self.root_instrument.visa_handle.read_raw()
+
+        # parse it
+        realdata = np.fromstring(rawdata, dtype='<i2')
+        numbers = realdata[::2]*2.0**(realdata[1::2]-124)
+
+        return numbers
 
 
 class ChannelBuffer(ArrayParameter):
@@ -20,9 +87,9 @@ class ChannelBuffer(ArrayParameter):
     def __init__(self, name: str, instrument: 'SR830', channel: int) -> None:
         """
         Args:
-            name (str): The name of the parameter
-            instrument (SR830): The parent instrument
-            channel (int): The relevant channel (1 or 2). The name should
+            name: The name of the parameter
+            instrument: The parent instrument
+            channel: The relevant channel (1 or 2). The name should
                 should match this.
         """
         self._valid_channels = (1, 2)
@@ -35,27 +102,28 @@ class ChannelBuffer(ArrayParameter):
             raise ValueError('Invalid parent instrument. ChannelBuffer '
                              'can only live on an SR830.')
 
-        super().__init__(name,
-                         shape=(1,),  # dummy initial shape
-                         unit='V',  # dummy initial unit
-                         setpoint_names=('Time',),
-                         setpoint_labels=('Time',),
-                         setpoint_units=('s',),
-                         docstring='Holds an acquired (part of the) '
-                                   'data buffer of one channel.')
+        super().__init__(
+            name,
+            shape=(1,),  # dummy initial shape
+            unit="V",  # dummy initial unit
+            setpoint_names=("Time",),
+            setpoint_labels=("Time",),
+            setpoint_units=("s",),
+            docstring="Holds an acquired (part of the) data buffer of one channel.",
+            instrument=instrument,
+        )
 
         self.channel = channel
-        self._instrument = instrument
 
-    def prepare_buffer_readout(self):
+    def prepare_buffer_readout(self) -> None:
         """
         Function to generate the setpoints for the channel buffer and
         get the right units
         """
-
-        N = self._instrument.buffer_npts()  # problem if this is zero?
+        assert isinstance(self.instrument, SR830)
+        N = self.instrument.buffer_npts()  # problem if this is zero?
         # TODO (WilliamHPNielsen): what if SR was changed during acquisition?
-        SR = self._instrument.buffer_SR()
+        SR = self.instrument.buffer_SR()
         if SR == 'Trigger':
             self.setpoint_units = ('',)
             self.setpoint_names = ('trig_events',)
@@ -70,7 +138,7 @@ class ChannelBuffer(ArrayParameter):
 
         self.shape = (N,)
 
-        params = self._instrument.parameters
+        params = self.instrument.parameters
         # YES, it should be: comparing to the string 'none' and not
         # the None literal
         if params[f'ch{self.channel}_ratio'].get() != 'none':
@@ -83,36 +151,39 @@ class ChannelBuffer(ArrayParameter):
                 self.unit = 'V'
 
         if self.channel == 1:
-            self._instrument._buffer1_ready = True
+            self.instrument._buffer1_ready = True
         else:
-            self._instrument._buffer2_ready = True
+            self.instrument._buffer2_ready = True
 
-    def get_raw(self):
+    def get_raw(self) -> ParamRawDataType:
         """
         Get command. Returns numpy array
         """
+        assert isinstance(self.instrument, SR830)
         if self.channel == 1:
-            ready = self._instrument._buffer1_ready
+            ready = self.instrument._buffer1_ready
         else:
-            ready = self._instrument._buffer2_ready
+            ready = self.instrument._buffer2_ready
 
         if not ready:
             raise RuntimeError('Buffer not ready. Please run '
                                'prepare_buffer_readout')
-        N = self._instrument.buffer_npts()
+        N = self.instrument.buffer_npts()
         if N == 0:
             raise ValueError('No points stored in SR830 data buffer.'
                              ' Can not poll anything.')
 
         # poll raw binary data
-        self._instrument.write(f'TRCL ? {self.channel}, 0, {N}')
-        rawdata = self._instrument.visa_handle.read_raw()
+        self.instrument.write(f"TRCL ? {self.channel}, 0, {N}")
+        rawdata = self.instrument.visa_handle.read_raw()
 
         # parse it
         realdata = np.fromstring(rawdata, dtype='<i2')
         numbers = realdata[::2]*2.0**(realdata[1::2]-124)
         if self.shape[0] != N:
-            raise RuntimeError("SR830 got {} points in buffer expected {}".format(N, self.shape[0]))
+            raise RuntimeError(
+                f"SR830 got {N} points in buffer expected {self.shape[0]}"
+            )
         return numbers
 
 
@@ -156,7 +227,7 @@ class SR830(VisaInstrument):
 
     _N_TO_INPUT_CONFIG = {v: k for k, v in _INPUT_CONFIG_TO_N.items()}
 
-    def __init__(self, name, address, **kwargs):
+    def __init__(self, name: str, address: str, **kwargs: Any):
         super().__init__(name, address, **kwargs)
 
         # Reference and phase
@@ -307,7 +378,7 @@ class SR830(VisaInstrument):
                                'on': 1,
                            })
 
-        def parse_offset_get(s):
+        def parse_offset_get(s: str) -> Tuple[float, int]:
             parts = s.split(',')
 
             return float(parts[0]), int(parts[1])
@@ -351,24 +422,6 @@ class SR830(VisaInstrument):
                                'GPIB': '1\n',
                            })
 
-        # Channel setup
-        for ch in range(1, 3):
-
-            # detailed validation and mapping performed in set/get functions
-            self.add_parameter(f'ch{ch}_ratio',
-                               label=f'Channel {ch} ratio',
-                               get_cmd=partial(self._get_ch_ratio, ch),
-                               set_cmd=partial(self._set_ch_ratio, ch),
-                               vals=Strings())
-            self.add_parameter(f'ch{ch}_display',
-                               label=f'Channel {ch} display',
-                               get_cmd=partial(self._get_ch_display, ch),
-                               set_cmd=partial(self._set_ch_display, ch),
-                               vals=Strings())
-            self.add_parameter(f'ch{ch}_databuffer',
-                               channel=ch,
-                               parameter_class=ChannelBuffer)
-
         # Data transfer
         self.add_parameter('X',
                            get_cmd='OUTP? 1',
@@ -390,6 +443,17 @@ class SR830(VisaInstrument):
                            get_parser=float,
                            unit='deg')
 
+        self.add_parameter(
+            "complex_voltage",
+            label="Voltage",
+            get_cmd=self._get_complex_voltage,
+            unit="V",
+            docstring="Complex voltage parameter "
+            "calculated from X, Y phase using "
+            "Z = X +j*Y",
+            vals=ComplexNumbers(),
+        )
+
         # Data buffer settings
         self.add_parameter('buffer_SR',
                            label='Buffer sample rate',
@@ -429,6 +493,33 @@ class SR830(VisaInstrument):
                            get_cmd='SPTS ?',
                            get_parser=int)
 
+        self.add_parameter('sweep_setpoints',
+                           parameter_class=GeneratedSetPoints,
+                           vals=Arrays(shape=(self.buffer_npts.get,)))
+
+        # Channel setup
+        for ch in range(1, 3):
+
+            # detailed validation and mapping performed in set/get functions
+            self.add_parameter(f'ch{ch}_ratio',
+                               label=f'Channel {ch} ratio',
+                               get_cmd=partial(self._get_ch_ratio, ch),
+                               set_cmd=partial(self._set_ch_ratio, ch),
+                               vals=Strings())
+            self.add_parameter(f'ch{ch}_display',
+                               label=f'Channel {ch} display',
+                               get_cmd=partial(self._get_ch_display, ch),
+                               set_cmd=partial(self._set_ch_display, ch),
+                               vals=Strings())
+            self.add_parameter(f'ch{ch}_databuffer',
+                               channel=ch,
+                               parameter_class=ChannelBuffer)
+            self.add_parameter(f'ch{ch}_datatrace',
+                               channel=ch,
+                               vals=Arrays(shape=(self.buffer_npts.get,)),
+                               setpoints=(self.sweep_setpoints,),
+                               parameter_class=ChannelTrace)
+
         # Auto functions
         self.add_function('auto_gain', call_cmd='AGAN')
         self.add_function('auto_reserve', call_cmd='ARSV')
@@ -483,7 +574,7 @@ class SR830(VisaInstrument):
             'r': '3',
             'p': '4',
         'phase': '4',
-           'θ' : '4',
+            'θ': '4',
          'aux1': '5',
          'aux2': '6',
          'aux3': '7',
@@ -505,19 +596,19 @@ class SR830(VisaInstrument):
         less than 100 ms.
 
         Args:
-            *parameters
-                From 2 to 6 strings of names of parameters for which the values
-                are requested. including: 'x', 'y', 'r', 'p', 'phase' or 'θ',
-                'aux1', 'aux2', 'aux3', 'aux4', 'freq', 'ch1', and 'ch2'.
+            *parameters: From 2 to 6 strings of names of parameters for which
+                the values are requested. including: 'x', 'y', 'r', 'p',
+                'phase' or 'θ', 'aux1', 'aux2', 'aux3', 'aux4', 'freq',
+                'ch1', and 'ch2'.
 
         Returns:
             A tuple of floating point values in the same order as requested.
 
         Examples:
-            lockin.snap('x','y') -> tuple(x,y)
+            >>> lockin.snap('x','y') -> tuple(x,y)
 
-            lockin.snap('aux1','aux2','freq','phase')
-                -> tuple(aux1,aux2,freq,phase)
+            >>> lockin.snap('aux1','aux2','freq','phase')
+            >>> -> tuple(aux1,aux2,freq,phase)
 
         Note:
             Volts for x, y, r, and aux 1-4
@@ -549,7 +640,7 @@ class SR830(VisaInstrument):
 
         return tuple(float(val) for val in output.split(','))
 
-    def increment_sensitivity(self):
+    def increment_sensitivity(self) -> bool:
         """
         Increment the sensitivity setting of the lock-in. This is equivalent
         to pushing the sensitivity up button on the front panel. This has no
@@ -560,7 +651,7 @@ class SR830(VisaInstrument):
         """
         return self._change_sensitivity(1)
 
-    def decrement_sensitivity(self):
+    def decrement_sensitivity(self) -> bool:
         """
         Decrement the sensitivity setting of the lock-in. This is equivalent
         to pushing the sensitivity down button on the front panel. This has no
@@ -571,7 +662,7 @@ class SR830(VisaInstrument):
         """
         return self._change_sensitivity(-1)
 
-    def _change_sensitivity(self, dn):
+    def _change_sensitivity(self, dn: int) -> bool:
         if self.input_config() in ['a', 'a-b']:
             n_to = self._N_TO_VOLT
             to_n = self._VOLT_TO_N
@@ -587,12 +678,13 @@ class SR830(VisaInstrument):
         self.sensitivity.set(n_to[n + dn])
         return True
 
-    def _set_buffer_SR(self, SR):
+    def _set_buffer_SR(self, SR: int) -> None:
         self.write(f'SRAT {SR}')
         self._buffer1_ready = False
         self._buffer2_ready = False
+        self.sweep_setpoints.update_units_if_constant_sample_rate()
 
-    def _get_ch_ratio(self, channel):
+    def _get_ch_ratio(self, channel: int) -> str:
         val_mapping = {1: {0: 'none',
                            1: 'Aux In 1',
                            2: 'Aux In 2'},
@@ -603,7 +695,7 @@ class SR830(VisaInstrument):
 
         return val_mapping[channel][resp]
 
-    def _set_ch_ratio(self, channel, ratio):
+    def _set_ch_ratio(self, channel: int, ratio: str) -> None:
         val_mapping = {1: {'none': 0,
                            'Aux In 1': 1,
                            'Aux In 2': 2},
@@ -613,12 +705,12 @@ class SR830(VisaInstrument):
         vals = val_mapping[channel].keys()
         if ratio not in vals:
             raise ValueError(f'{ratio} not in {vals}')
-        ratio = val_mapping[channel][ratio]
+        ratio_int = val_mapping[channel][ratio]
         disp_val = int(self.ask(f'DDEF ? {channel}').split(',')[0])
-        self.write(f'DDEF {channel}, {disp_val}, {ratio}')
+        self.write(f'DDEF {channel}, {disp_val}, {ratio_int}')
         self._buffer_ready = False
 
-    def _get_ch_display(self, channel):
+    def _get_ch_display(self, channel: int) -> str:
         val_mapping = {1: {0: 'X',
                            1: 'R',
                            2: 'X Noise',
@@ -633,7 +725,7 @@ class SR830(VisaInstrument):
 
         return val_mapping[channel][resp]
 
-    def _set_ch_display(self, channel, disp):
+    def _set_ch_display(self, channel: int, disp: str) -> None:
         val_mapping = {1: {'X': 0,
                            'R': 1,
                            'X Noise': 2,
@@ -647,20 +739,30 @@ class SR830(VisaInstrument):
         vals = val_mapping[channel].keys()
         if disp not in vals:
             raise ValueError(f'{disp} not in {vals}')
-        disp = val_mapping[channel][disp]
+        disp_int = val_mapping[channel][disp]
         # Since ratio AND display are set simultaneously,
         # we get and then re-set the current ratio value
         ratio_val = int(self.ask(f'DDEF ? {channel}').split(',')[1])
-        self.write(f'DDEF {channel}, {disp}, {ratio_val}')
+        self.write(f'DDEF {channel}, {disp_int}, {ratio_val}')
         self._buffer_ready = False
+        # we update the unit of the datatrace
+        # according to the choice of channel
+        params = self.parameters
+        dataparam = params[f'ch{channel}_datatrace']
+        assert isinstance(dataparam, ChannelTrace)
+        dataparam.update_unit()
 
-    def _set_units(self, unit):
+    def _set_units(self, unit: str) -> None:
         # TODO:
         # make a public parameter function that allows to change the units
         for param in [self.X, self.Y, self.R, self.sensitivity]:
             param.unit = unit
 
-    def _get_input_config(self, s):
+    def _get_complex_voltage(self) -> complex:
+        x, y = self.snap("X", "Y")
+        return x + 1.0j * y
+
+    def _get_input_config(self, s: int) -> str:
         mode = self._N_TO_INPUT_CONFIG[int(s)]
 
         if mode in ['a', 'a-b']:
@@ -672,7 +774,7 @@ class SR830(VisaInstrument):
 
         return mode
 
-    def _set_input_config(self, s):
+    def _set_input_config(self, s: str) -> int:
         if s in ['a', 'a-b']:
             self.sensitivity.vals = self._VOLT_ENUM
             self._set_units('V')
@@ -682,14 +784,92 @@ class SR830(VisaInstrument):
 
         return self._INPUT_CONFIG_TO_N[s]
 
-    def _get_sensitivity(self, s):
+    def _get_sensitivity(self, s: int) -> float:
         if self.input_config() in ['a', 'a-b']:
             return self._N_TO_VOLT[int(s)]
         else:
             return self._N_TO_CURR[int(s)]
 
-    def _set_sensitivity(self, s):
+    def _set_sensitivity(self, s: float) -> int:
         if self.input_config() in ['a', 'a-b']:
             return self._VOLT_TO_N[s]
         else:
             return self._CURR_TO_N[s]
+
+    def autorange(self, max_changes: int = 1) -> None:
+        """
+        Automatically changes the sensitivity of the instrument according to
+        the R value and defined max_changes.
+
+        Args:
+            max_changes: Maximum number of steps allowing the function to
+                automatically change the sensitivity (default is 1). The actual
+                number of steps needed to change to the optimal sensitivity may
+                be more or less than this maximum.
+        """
+        def autorange_once() -> bool:
+            r = self.R()
+            sens = self.sensitivity()
+            if r > 0.9 * sens:
+                return self.increment_sensitivity()
+            elif r < 0.1 * sens:
+                return self.decrement_sensitivity()
+            return False
+
+        sets = 0
+        while autorange_once() and sets < max_changes:
+            sets += 1
+            time.sleep(self.time_constant())
+
+    def set_sweep_parameters(self,
+                             sweep_param: Parameter,
+                             start: float,
+                             stop: float,
+                             n_points: int = 10,
+                             label: Union[str, None] = None) -> None:
+
+        self.sweep_setpoints.sweep_array = np.linspace(start, stop, n_points)
+        self.sweep_setpoints.unit = sweep_param.unit
+        if label is not None:
+            self.sweep_setpoints.label = label
+        elif sweep_param.label is not None:
+            self.sweep_setpoints.label = sweep_param.label
+
+
+class GeneratedSetPoints(Parameter):
+    """
+    A parameter that generates a setpoint array from start, stop and num points
+    parameters.
+    """
+    def __init__(self,
+                 sweep_array: Iterable[Union[float, int]] = np.linspace(0, 1, 10),
+                 *args: Any, **kwargs: Any) -> None:
+        super().__init__(*args, **kwargs)
+        self.sweep_array = sweep_array
+        self.update_units_if_constant_sample_rate()
+
+    def update_units_if_constant_sample_rate(self) -> None:
+        """
+        If the buffer is filled at a constant sample rate,
+        update the unit to "s" and label to "Time";
+        otherwise do nothing
+        """
+        assert isinstance(self.root_instrument, SR830)
+        SR = self.root_instrument.buffer_SR.get()
+        if SR != 'Trigger':
+            self.unit = 's'
+            self.label = 'Time'
+
+    def set_raw(self, value: Iterable[Union[float, int]]) -> None:
+        self.sweep_array = value
+
+    def get_raw(self) -> ParamRawDataType:
+        assert isinstance(self.root_instrument, SR830)
+        SR = self.root_instrument.buffer_SR.get()
+        if SR == 'Trigger':
+            return self.sweep_array
+        else:
+            N = self.root_instrument.buffer_npts.get()
+            dt = 1/SR
+
+            return np.linspace(0, N*dt, N)
